﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.Extensions.Options;
using SixLabors.ImageSharp;
using SixLabors.ImageSharp.PixelFormats;
using SixLabors.Primitives;

namespace ImagEncode.Services {
    public class BinaryCodePictureGenerator<TPixel> : ICodePictureGenerator<TPixel> where TPixel : struct, IPixel<TPixel> {
        private const int HeaderOffset = 2; // Length of header

        private readonly BinaryCodePictureGeneratorOptions<TPixel> options;

        public BinaryCodePictureGenerator(IOptions<BinaryCodePictureGeneratorOptions<TPixel>> options) {
            this.options = options.Value;
        }

        public Image<TPixel> CreateImage(string text, Size size) {
            if (text == null) throw new ArgumentNullException(nameof(text));
            if (string.IsNullOrWhiteSpace(text)) throw new ArgumentException("Value cannot be empty or whitespace only string.", nameof(text));

            // Convert string to UTF8 byte array
            var textData = System.Text.Encoding.UTF8.GetBytes(text);
            var textBits = new BitArray(textData);

            // Check size
            if (size.Width % this.options.ModuleSize.Width > 0) throw new Exception("Image width must be divisible by module width.");

            // Prepare image and write header
            var img = new Image<TPixel>(Configuration.Default, size.Width, size.Height, this.options.BackgroundColor);
            this.DrawModule(img, 0, 0, this.options.ForegroundColor);
            this.DrawModule(img, 0, 1, this.options.BackgroundColor);

            // Write all bits
            for (var i = 0; i < textBits.Length; i++) {
                var col = (i + HeaderOffset) % (img.Width / this.options.ModuleSize.Width);
                var row = (i + HeaderOffset) / (img.Width / this.options.ModuleSize.Width);
                var result = this.DrawModule(img, row, col, textBits[i] ? this.options.ForegroundColor : this.options.BackgroundColor);
                if (!result) break;
            }

            return img;
        }

        public bool DrawModule(Image<TPixel> img, int row, int col, TPixel color) {
            var x1 = col * this.options.ModuleSize.Width;
            var x2 = x1 + this.options.ModuleSize.Width;
            var y1 = row * (this.options.ModuleSize.Height + this.options.VerticalSpacing);
            var y2 = y1 + this.options.ModuleSize.Height;

            //if (x2 >= img.Width || y2 >= img.Height) return false;

            for (var x = x1; x < x2; x++) {
                for (var y = y1; y < y2; y++) {
                    img[x, y] = color;
                }
            }

            return true;
        }
    }

}